#include <stdio.h>
#include <stdlib.h>
#include "pri.h"


#define GETBYTE(input)         fgetc((FILE*)(input))
#define PUTBYTE(output,byte)   fprintf((output), "%c%c%c%c%c%c%c%c", \
                                       ((byte) & 0x80) ? '*' : ' ',  \
                                       ((byte) & 0x40) ? '*' : ' ',  \
                                       ((byte) & 0x20) ? '*' : ' ',  \
                                       ((byte) & 0x10) ? '*' : ' ',  \
                                       ((byte) & 0x08) ? '*' : ' ',  \
                                       ((byte) & 0x04) ? '*' : ' ',  \
                                       ((byte) & 0x02) ? '*' : ' ',  \
                                       ((byte) & 0x01) ? '*' : ' ')

#include "unrle.c"


/* pri_readheader()
 * Helper function that reads a bitmap header byte-for-byte and optionally
 * aligns the multi-byte fields to little Endian
 */
int pri_readheader(FILE *input, PRHDR *pri)
{
    unsigned char header[PRHEADERSIZE];
    unsigned char *ptr = header;
    int c;

    while ((ptr - header) < PRHEADERSIZE && (c = GETBYTE(input)) != EOF)
        *ptr++ = (unsigned char)c;

    pri->size   = PRHDR_SIZE(header);
    pri->id     = PRHDR_ID(header);
    pri->layout = PRHDR_LAYOUT(header);
    pri->width  = PRHDR_WIDTH(header);
    pri->height = PRHDR_HEIGHT(header);
    pri->depth  = PRHDR_DEPTH(header);

    if (pri->size == 0 || pri->id != PRSIGNATURE)
        return 0;

    return 1;
}

/* pri_show()
 * Writes the image as a kind of ASCII art. The Poly-Raster bitmap must be
 * 1-bpp for this routine (and have lay-out 0).
 */
void pri_show(FILE *input, FILE *output, int width, int height)
{
    struct rlestat stat = { 0 };
    int x, y, c;

    for (y = 0; y < height; y++) {
      for (x = 0; x < width; x += 8) {
        c = unrle(input, &stat);
        PUTBYTE(output, c);
      } /* for */
      fprintf(output, "\n");
    } /* for */
}

int main(int argc, char *argv[])
{
    FILE *input, *output;
    PRHDR pri;

    /* open files */
    input = (argc >= 2) ? fopen(argv[1], "rb") : stdin;
    output = (argc >= 3) ? fopen(argv[2], "wb") : stdout;
    if (input == NULL) {
        perror("Input file");
        return 1;
    }
    if (output == NULL) {
        perror("Output file");
        return 1;
    }

    /* stream in a header */
    for ( ;; ) {
        if (!pri_readheader(input, &pri)) {
            fprintf(stderr, "No supported bitmap type found\n");
            return 1;
        }
        if (pri.layout == 0 && pri.depth == 1)
            break;
        /* ignore this bitmap */
        while (pri.size > PRHEADERSIZE && GETBYTE(input) != EOF)
            pri.size--;
    }

    /* write the PPM image */
    pri_show(input, output, pri.width, pri.height);

    /* clean up */
    if (input != stdin)
        fclose(input);
    if (output != stdout)
        fclose(output);

    return 0;
}
