#include <stdio.h>
#include <stdlib.h>
#include "pri.h"


#define GETBYTE(input)        fgetc((FILE*)(input))
#define PUTBYTE(output,byte)  fputc((byte),(output))

#include "unrle.c"


/* pri_readheader()
 * Helper function that reads a bitmap header byte-for-byte and optionally
 * aligns the multi-byte fields to little Endian
 */
int pri_readheader(FILE *input, PRHDR *pri)
{
    unsigned char header[PRHEADERSIZE];
    unsigned char *ptr = header;
    int c;

    while ((ptr - header) < PRHEADERSIZE && (c = GETBYTE(input)) != EOF)
        *ptr++ = (unsigned char)c;

    pri->size   = PRHDR_SIZE(header);
    pri->id     = PRHDR_ID(header);
    pri->layout = PRHDR_LAYOUT(header);
    pri->width  = PRHDR_WIDTH(header);
    pri->height = PRHDR_HEIGHT(header);
    pri->depth  = PRHDR_DEPTH(header);

    if (pri->size == 0 || pri->id != PRSIGNATURE)
        return 0;

    return 1;
}

/* write_ppm()
 * Writes the header of a PBM image, then copies pixels from input to output.
 * For a PBM file, 0 is white and 1 is black; for a PRI file, the inverse is
 * true. So, we invert all bytes before writing them.
 */
void write_ppm(FILE *input, FILE *output, int width, int height)
{
    struct rlestat stat = { 0 };
    long size = height * ((width + 7) / 8);
    int c;

    fprintf(output, "P4\n%d %d\n", width, height);

    while (size-- > 0) {
        c = unrle(input, &stat);
        PUTBYTE(output, ~c);
    } /* while */
}

int main(int argc, char *argv[])
{
    FILE *input, *output;
    PRHDR pri;

    /* open files */
    input = (argc >= 2) ? fopen(argv[1], "rb") : stdin;
    output = (argc >= 3) ? fopen(argv[2], "wb") : stdout;
    if (input == NULL) {
        perror("Input file");
        return 1;
    }
    if (output == NULL) {
        perror("Output file");
        return 1;
    }

    /* stream in a header */
    for ( ;; ) {
        if (!pri_readheader(input, &pri)) {
            fprintf(stderr, "No supported bitmap type found\n");
            return 1;
        }
        if (pri.layout == 0 && pri.depth == 1)
            break;
        /* ignore this bitmap */
        while (pri.size > PRHEADERSIZE && GETBYTE(input) != EOF)
            pri.size--;
    }

    /* write the PPM image */
    write_ppm(input, output, pri.width, pri.height);

    /* clean up */
    if (input != stdin)
        fclose(input);
    if (output != stdout)
        fclose(output);

    return 0;
}
