#include <stdio.h>
#include <stdlib.h>
#include "pri.h"

/* Macros to help decoding the pixel data (Run-Length Encoded) */
#define GETBYTE(intput)       (*((unsigned char**)input)+=1, *(*((unsigned char**)input)-1))
#define PUTBYTE(output,byte)  fputc((byte),(output))

#include "unrle.c"


/* pri_readheader()
 * Helper function that reads a bitmap header and checks whether it is valid
 */
int pri_readheader(FILE *file, PRHDR *pri)
{
    unsigned char header[PRHEADERSIZE];

    if (fread(header, 1, PRHEADERSIZE, file) < PRHEADERSIZE)
        return 0;

    pri->size   = PRHDR_SIZE(header);
    pri->id     = PRHDR_ID(header);
    pri->layout = PRHDR_LAYOUT(header);
    pri->width  = PRHDR_WIDTH(header);
    pri->height = PRHDR_HEIGHT(header);
    pri->depth  = PRHDR_DEPTH(header);
    if (pri->size == 0 || pri->id != PRSIGNATURE)
        return 0;

    return 1;
}

/* write_ppm()
 * Writes the header of a PBM image, then copies pixels from input to output.
 * For a PBM file, 0 is white and 1 is black; for a PRI file, the inverse is
 * true. So, we invert all bytes before writing them.
 */
void write_ppm(FILE *file, int width, int height, unsigned char *data)
{
    struct rlestat stat = { 0 };
    long size = height * ((width + 7) / 8);
    int c;


    fprintf(file, "P4\n%d %d\n", width, height);

    while (size-- > 0) {
        c = unrle(&data, &stat);
        PUTBYTE(file, ~c);
    } /* while */
}

int main(int argc, char *argv[])
{
    FILE *input, *output;
    unsigned char *image;
    PRHDR pri;

    if (argc != 3)
        fprintf(stderr, "usage: %s <input> <output>\n", argv[0]);

    /* open files */
    input = fopen(argv[1], "rb");
    output = fopen(argv[2], "wb");
    if (input == NULL) {
        perror("Input file");
        return 1;
    }
    if (output == NULL) {
        perror("Output file");
        return 1;
    }

    /* browse through the file to find the matching bitmap */
    for ( ;; ) {
        if (!pri_readheader(input, &pri)) {
            fprintf(stderr, "No supported bitmap type found\n");
            return 1;
        }
        if (pri.layout == 0 && pri.depth == 1)
            break;
        fseek(input, pri.size - sizeof pri, SEEK_CUR);
    }

    /* allocate memory */
    image = malloc(pri.size - sizeof pri);
    if (image == NULL) {
        fprintf(stderr, "Insufficient memory\n");
        return 1;
    }

    /* read in the entire source bitmap */
    fread(image, 1, pri.size - sizeof pri, input);

    /* write the PPM image */
    write_ppm(output, pri.width, pri.height, image);

    /* clean up */
    free(image);
    fclose(input);
    fclose(output);
    return 0;
}
